local Version = "1.0"
local Loaded = false

local AltsMail_OrigGetAutoCompleteResults = nil
local RealmName
local CharName
local MyAlts = {}
local RealmNameCompressedToNormal = {}

local menu
local expandButton

local settingsFrame = nil
local searchAlts, searchGroup, searchGuild, searchFriend, searchBnet, searchInteracted, searchOnline, searchAll
local searchFlag = AUTOCOMPLETE_FLAG_ALL
local mailOpen = false
local haveMail = false

local AUTOCOMPLETE_FLAG_ALTS 			= 0x100

local function OnCharHandler(s)

	local name = s:GetText()

	if ( strlen(name) <= 0 ) then
		return
	end

	local t = (
		AltsMail_GetAutoCompleteAltResults(name) or
		AltsMail_OrigGetAutoCompleteResults(name, searchFlag, 0, 1)
	)
	if (t) and (#t > 0) then
		completedName = t[1].name
	end

	local completedNameRemainderPosition = s:GetCursorPosition()
	if completedName then
		s:SetText(completedName)
		s:HighlightText(completedNameRemainderPosition, strlen(completedName))
	end

	-- to keep the new auto complete functionality
	AutoComplete_Update(s, name, completedNameRemainderPosition)
	SendMailFrame_Update()
end

local function comp_func(t1, t2)
	return t1.name < t2.name
end

local function NewGetAutoCompleteResults(t, include, exclude, maxResults, ...)
	local result
	if not(mailOpen) then
		result = AltsMail_OrigGetAutoCompleteResults(t, include, exclude, maxResults, ...)
		return result
	else
		result = AltsMail_OrigGetAutoCompleteResults(t, searchFlag, exclude, maxResults, ...)
		--[[for k, v in pairs(result) do
			print(k, v)
			if type(v) == 'table' then
				for pk, pv in pairs(v) do
					print(pk, pv)
				end
			end
		end]]
	end

	local k, v, ik, iv
	if (searchAlts) then
		local r = AltsMail_GetAutoCompleteAltResults(t)
		for k, v in ipairs(r) do
			local found = false
			for ik, iv in ipairs(result) do
				if ( v.name == iv.name ) then
					found = true
				end
			end

			if not(found) then
				table.insert(result, v)
			end
		end
	end

	table.sort(result, comp_func)

	local i = table.getn(result)
	while ( i > maxResults ) do
		table.remove(result)
		i=i-1
	end

	return result
end

function AltsMail_GetAutoCompleteAltResults(t)

	local result = {}

	local text = string.lower(t)
	local textlen = strlen(text)

	for nm, rname in pairs(MyAlts) do
		if (( strlen(nm) >= textlen ) and ( string.sub(string.lower(nm), 1, textlen) == text )) then
			for _, _rname in ipairs(rname) do
				if _rname ~= RealmName then
					table.insert(result, {name = nm..'-'.._rname:gsub(" ", ""), priority = LE_AUTOCOMPLETE_PRIORITY_OTHER})
				else
					table.insert(result, {name = nm, priority = LE_AUTOCOMPLETE_PRIORITY_OTHER})
				end
			end
		end
	end

	--[[for k, v in pairs(result) do
		print('results', k)
		for _k, _v in pairs(v) do
			print(_k, _v)
		end
	end]]
	return result
end

local function SetSearchFlag()
	searchFlag = 0
	if searchAll then
		searchFlag = searchFlag + AUTOCOMPLETE_FLAG_ALL
		return searchFlag
	end
	
	if searchAlts then
		searchFlag = searchFlag + AUTOCOMPLETE_FLAG_ALTS
	end
	if searchGroup then
		searchFlag = searchFlag + AUTOCOMPLETE_FLAG_IN_GROUP
	end
	if searchGuild then
		searchFlag = searchFlag + AUTOCOMPLETE_FLAG_IN_GUILD
	end
	if searchFriend then
		searchFlag = searchFlag + AUTOCOMPLETE_FLAG_FRIEND
	end
	if searchBnet then
		searchFlag = searchFlag + AUTOCOMPLETE_FLAG_BNET
	end
	if searchInteracted then
		searchFlag = searchFlag + AUTOCOMPLETE_FLAG_INTERACTED_WITH
	end
	if searchOnline then
		searchFlag = searchFlag + AUTOCOMPLETE_FLAG_ONLINE
	end
	return searchFlag
end

local function ParseSearchFlag(flag)
	if searchFlag == AUTOCOMPLETE_FLAG_ALL then
		searchAll = true
	end
	searchAlts = bit.band(flag, AUTOCOMPLETE_FLAG_ALTS) ~= 0
	searchGroup = bit.band(flag, AUTOCOMPLETE_FLAG_IN_GROUP) ~= 0
	searchGuild = bit.band(flag, AUTOCOMPLETE_FLAG_IN_GUILD) ~= 0
	searchFriend = bit.band(flag, AUTOCOMPLETE_FLAG_FRIEND) ~= 0
	searchBnet = bit.band(flag, AUTOCOMPLETE_FLAG_BNET) ~= 0
	searchInteracted = bit.band(flag, AUTOCOMPLETE_FLAG_INTERACTED_WITH) ~= 0
	searchOnline = bit.band(flag, AUTOCOMPLETE_FLAG_ONLINE) ~= 0
end

local function AltsMailSettings_MenuItemChecked(self)
	--print(self.value.." checked "..tostring(self.checked))
	if self.value == "Alts" then
		searchAlts = self.checked
	end
	if self.value == "Group" then
		searchGroup = self.checked
	end
	if self.value == "Guild" then
		searchGuild = self.checked
	end
	if self.value == "Friend" then
		searchFriend = self.checked
	end
	if self.value == "Bnet" then
		searchBnet = self.checked
	end
	if self.value == "Interacted" then
		searchInteracted = self.checked
	end
	if self.value == "Online" then
		searchOnline = self.checked
	end
	if self.value == "All" then
		searchAll = self.checked
	end

	SetSearchFlag()
	--print(string.format("flag %X", searchFlag))

	AltsMail_Alts["searchFlag"] = searchFlag
end

local function expandButton_OnEnter(self)
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	if (self.expanded) then
		GameTooltip:SetText(HIGHLIGHT_FONT_COLOR_CODE..self.collapseTooltip..FONT_COLOR_CODE_CLOSE)
	else
		GameTooltip:SetText(HIGHLIGHT_FONT_COLOR_CODE..self.expandTooltip..FONT_COLOR_CODE_CLOSE)
	end
end

local function expandButton_OnLeave(self)
	GameTooltip:Hide()
end

local function setExpandButtonTextures(self)
	if (self.expanded) then
		self:SetNormalTexture([[Interface\Buttons\UI-SpellbookIcon-PrevPage-Up]])
		self:SetPushedTexture([[Interface\Buttons\UI-SpellbookIcon-PrevPage-Down]])
		self:SetDisabledTexture([[Interface\Buttons\UI-SpellbookIcon-PrevPage-Disabled]])
	else
		self:SetNormalTexture([[Interface\Buttons\UI-SpellbookIcon-NextPage-Up]])
		self:SetPushedTexture([[Interface\Buttons\UI-SpellbookIcon-NextPage-Down]])
		self:SetDisabledTexture([[Interface\Buttons\UI-SpellbookIcon-NextPage-Disabled]]) 		
	end
end

local function expandButton_OnClick(self)
	if (self.expanded) then
		PlaySound("igCharacterInfoClose")
	else
		PlaySound("igCharacterInfoOpen")
	end
	self.expanded = not(self.expanded)
	setExpandButtonTextures(self)
	if (GameTooltip:GetOwner() == self) then
		self:GetScript("OnEnter")(self)
	end

	if (self.expanded) then
		menu[2].checked = searchAlts
		menu[3].checked = searchGroup
		menu[4].checked = searchGuild
		menu[5].checked = searchFriend
		menu[6].checked = searchBnet
		menu[7].checked = searchInteracted
		menu[8].checked = searchOnline
		menu[9].checked = searchAll
		EasyMenu(menu, self, expandButton, 0, 0, nil)
	else
		CloseDropDownMenus(1)
	end
end

-- list

local function AltsMail_ShowList()
	--expandButton.expanded = false
	--setExpandButtonTextures(expandButton)
	AltsMailList_Frame:ClearAllPoints()
	AltsMailList_Frame:SetPoint("TOPLEFT", MailFrame, "TOPRIGHT", 30, 0)
	--AltsMailList_Frame:SetPoint("BOTTOM", MailFrame, "BOTTOM", 0, 0)
	AltsMailList_Frame:SetHeight(MailFrame:GetHeight())
	AltsMailList_Frame:Show()
	AltsMailList_Frame:SetFrameStrata('LOW')
	SetPortraitToTexture(_G[AltsMailList_Frame:GetName().."Portrait"], 'Interface\\MailFrame\\Mail-Icon')
	AltsMailList_Frame:Show()
	AltsMailList_Update()
end

local function AltsMail_HideList()
	AltsMailList_Frame:Hide()
end

local function OnSendMail()
	local target = SendMailNameEditBox:GetText()
	-- split string by '-'
	local values = {strsplit("-", target)}
	if #values > 0 then
		local name = values[1]
		-- get realm
		local rname
		if #values > 1 then
			rname = RealmNameCompressedToNormal[values[2]]
		end
		if not(rname) then
			rname = RealmName
		end
		-- now set mail expected
		--print(rname, name)
		if AltsMail_Alts[rname] and AltsMail_Alts[rname][name] then
			AltsMail_Alts[rname][name].haveMail = true
			if AltsMailList_Frame:IsVisible() then
				AltsMailList_Update()
			end
		end
	end
end

local function OnAhBid()
	AltsMail_Alts[RealmName][CharName].haveAH = true
end

local function OnAhCreate()
	AltsMail_Alts[RealmName][CharName].haveAH = true
end

local function initialize()
	if ( Loaded ) then
		return
	end

	AltsMailList_Frame = CreateFrame("Frame", "AltsMailListFrame", MailFrame, "AltsMailList_Template")
	AltsMailList_Frame.sf = _G[AltsMailList_Frame:GetName().."ScrollFrame"]
	AltsMailList_Frame.sf.buttons = {}
	AltsMailList_SetListButtons()

	RealmName = GetRealmName()
	CharName = UnitName("player")
	local _, classFileName = UnitClass("player")

	AltsMail_Alts = AltsMail_Alts or {}
	AltsMail_Alts[RealmName] = AltsMail_Alts[RealmName] or {}

	-- add yourself if not exists
	local now = time()
	if type(AltsMail_Alts[RealmName][CharName]) ~= 'table' then
		AltsMail_Alts[RealmName][CharName] = nil
	end
	AltsMail_Alts[RealmName][CharName] = AltsMail_Alts[RealmName][CharName] or {}
	--print(RealmName, CharName, now)
	AltsMail_Alts[RealmName][CharName].lastLogin = now
	AltsMail_Alts[RealmName][CharName].classFileName = classFileName
	
	-- get alt names
	for rname, v in pairs(AltsMail_Alts) do
		if rname ~= 'searchFlag' then
			for name, _ in pairs(v) do
				if (v[name]) and not(name == CharName and rname == RealmName) then
					MyAlts[name] = MyAlts[name] or {}
					MyAlts[name][#MyAlts[name] + 1] = rname
				end
			end
			local rname_nospace = rname:gsub(" ", "")
			RealmNameCompressedToNormal[rname_nospace] = rname
		end
	end

	AltsMail_OrigGetAutoCompleteResults = GetAutoCompleteResults
	GetAutoCompleteResults = NewGetAutoCompleteResults

	SendMailNameEditBox:SetScript("OnTextChanged", nil)
	SendMailNameEditBox:SetScript("OnChar", OnCharHandler)
	SendMailMailButton:HookScript("OnClick", OnSendMail)
	SendMailFrame:HookScript("OnShow", AltsMail_ShowList)
	SendMailFrame:HookScript("OnHide", AltsMail_HideList)

	AltsMail_Alts["searchFlag"] = AltsMail_Alts["searchFlag"] or AUTOCOMPLETE_FLAG_ALL
	searchFlag = AltsMail_Alts["searchFlag"]
	
	ParseSearchFlag(searchFlag)

	if not(expandButton) then
		expandButton = CreateFrame("Button", "AltsMailExpandButton", SendMailFrame)
		expandButton:SetWidth(32)
		expandButton:SetHeight(32)
		
		expandButton.expanded = false
		
		setExpandButtonTextures(expandButton)
		expandButton:SetHighlightTexture([[Interface\Buttons\UI-Common-MouseHilight]])
		expandButton.collapseTooltip = "Hide filters"
		expandButton.expandTooltip = "Show filters"
		expandButton:SetScript("OnEnter", expandButton_OnEnter)
		expandButton:SetScript("OnLeave", expandButton_OnLeave)
		expandButton:SetScript("OnClick", expandButton_OnClick)
		
		expandButton:ClearAllPoints()
		expandButton:SetPoint("BOTTOMLEFT", SendMailNameEditBox, "BOTTOMRIGHT", 0, 0)
		expandButton:SetFrameLevel(expandButton:GetParent():GetFrameLevel() + 2)
	end
	
	menu = menu or {
			{ text = "Search filters", isTitle = true, notCheckable = true, justifyH = "CENTER"},
			{ text = "Alts", value = text, func = AltsMailSettings_MenuItemChecked, checked = searchAlts, keepShownOnClick = true },
			{ text = "Group", value = text, func = AltsMailSettings_MenuItemChecked, checked = searchGroup, keepShownOnClick = true },
			{ text = "Guild", value = text, func = AltsMailSettings_MenuItemChecked, checked = searchGuild, keepShownOnClick = true },
			{ text = "Friend", value = text, func = AltsMailSettings_MenuItemChecked, checked = searchFriend, keepShownOnClick = true },
			{ text = "Bnet", value = text, func = AltsMailSettings_MenuItemChecked, checked = searchBnet, keepShownOnClick = true },
			{ text = "Interacted", value = text, func = AltsMailSettings_MenuItemChecked, checked = searchInteracted, keepShownOnClick = true },
			{ text = "Online", value = text, func = AltsMailSettings_MenuItemChecked, checked = searchOnline, keepShownOnClick = true },
			{ text = "All", value = text, func = AltsMailSettings_MenuItemChecked, checked = searchAll, keepShownOnClick = true },
		}

	haveMail = HasNewMail()
	AltsMail_Alts[RealmName][CharName].haveMail = haveMail
	
	-- todo use duration
	AltsMail_Alts[RealmName][CharName].haveAH = nil
	
	Loaded = true
end

local function OnMailShow()
	SendMailCostMoneyFrame:Hide()
	local width = 200
	SendMailNameEditBox:SetWidth(width)
	SendMailNameEditBoxMiddle:SetWidth(width - 9)
end

local function EventHandler(self, event, ...)

	if ( event == "ADDON_LOADED" ) then
		local addon = ...
		if ( addon == nil ) then
			return
		end
			
		if ( addon == "AltsMail" ) then
			self:UnregisterEvent("ADDON_LOADED")
			initialize()
		end

	elseif event == "MAIL_SHOW" then
		mailOpen = true
		OnMailShow()
	elseif event == "MAIL_CLOSED" then
		mailOpen = false
		if expandButton.expanded then
			expandButton_OnClick(expandButton)
		end
		AltsMailList_Frame:Hide()
	elseif event == "UPDATE_PENDING_MAIL" then
		haveMail = HasNewMail()
		if Loaded then
			AltsMail_Alts[RealmName][CharName].haveMail = haveMail
		end
	elseif event == "AUCTION_HOUSE_SHOW" then
		self:UnregisterEvent("AUCTION_HOUSE_SHOW")
		-- AH AuctionFrameBid
		BidBidButton:HookScript("OnClick", OnAhBid)
		-- AH sell
		-- DurationDropDown, AuctionFrameAuctions.duration
		AuctionsCreateAuctionButton:HookScript("OnClick", OnAhCreate)
	end
end

-- main
local frame = CreateFrame("Frame")
frame:RegisterEvent("ADDON_LOADED")
frame:RegisterEvent("MAIL_SHOW")
frame:RegisterEvent("MAIL_CLOSED")
frame:RegisterEvent("UPDATE_PENDING_MAIL")
frame:RegisterEvent("AUCTION_HOUSE_SHOW")
frame:SetScript("OnEvent", EventHandler)

